/**
 * Copyright 2006-2007 by jFramework.org.
 */
package org.jframework.core.util;

import junit.framework.TestCase;

import java.util.Arrays;
import java.util.Collections;

import org.apache.log4j.BasicConfigurator;

/**
 * Tests string utilities
 *
 * @author mgridnev
 * @version 2.0, Feb 21, 2007
 */
public class StringUtilsTest extends TestCase
{
    protected void setUp() throws Exception
    {
        BasicConfigurator.configure();
    }

    protected void tearDown() throws Exception
    {
        BasicConfigurator.resetConfiguration();
    }

    /**
     * Tests truncation of a string
     */
    public void testTruncate()
    {
        final String input = "The quick brown fox";

        assertEquals("", StringUtils.truncate("", 0, 0));
        assertEquals("", StringUtils.truncate("", 5, 5));

        assertEquals(input, StringUtils.truncate(input, input.length(), 0));
        assertEquals(input, StringUtils.truncate(input, 0, input.length()));
        assertEquals(input, StringUtils.truncate(input, input.length(), input.length()));
        assertEquals(input, StringUtils.truncate(input, input.length() / 2, input.length() - input.length() / 2));
        assertEquals("...", StringUtils.truncate(input, 0, 0));

        assertEquals("The q...n fox", StringUtils.truncate(input, 5, 5));
        assertEquals("...n fox", StringUtils.truncate(input, 0, 5));
        assertEquals("The q...", StringUtils.truncate(input, 5, 0));
    }

    /**
     * Test padding a string
     */
    public void testPad()
    {
        assertEquals("abc   ", StringUtils.pad("abc", ' ', 6));
        assertEquals("abc", StringUtils.pad("abc", ' ', 3));
        assertEquals("abc", StringUtils.pad("abc", ' ', 2));
    }

    public void testEscape()
    {
        final String string = "one, two, three.";
        assertEquals("one\\, two\\, three\\.", StringUtils.escape(string, ",.", '\\'));
    }

    public void testReplaceAll()
    {
        assertEquals("", StringUtils.replaceAll("", ",", "\\,"));

        assertEquals("item1\\, item2", StringUtils.replaceAll("item1, item2", ",", "\\,"));
        assertEquals("\\, item1\\, item2\\, item3\\,", StringUtils.replaceAll(", item1, item2, item3,", ",", "\\,"));
    }

    public void testCharsToString()
    {
        assertEquals("", StringUtils.toString('\0', 0));
        assertEquals("aaa", StringUtils.toString('a', 3));
    }

    public void testToString()
    {
        assertEquals("", StringUtils.toString(Collections.EMPTY_LIST, ", ", true, null));
        assertEquals(StringUtils.NULL, StringUtils.toString(Arrays.asList((String) null), ", ", true, null));

        assertEquals("item", StringUtils.toString(Arrays.asList("item"), ", ", true, null));
        assertEquals("item, item2", StringUtils.toString(Arrays.asList("item", "item2"), ", ", true, null));

        assertEquals("item, item\\, 2", StringUtils.toString(Arrays.asList("item", "item, 2"), ", ", true, null));
        assertEquals(StringUtils.toString(Arrays.asList("item", "item, 2"), ", ", false, null), "item, item, 2");
    }

    public void testToPlainText()
    {
        final String input = "<html><body>Line one<br>Line two</body></html>";

        assertEquals("Line one\nLine two", StringUtils.toPlainText(input));
    }

    public void testToHTML()
    {
        assertEquals("<html><body>line &lt;one&gt;<br>&amp;line &quot;two&quot;</body></html>", StringUtils.toHTML("line <one>\n&line \"two\""));
    }

    public void testNormalizedHTML()
    {
        assertEquals("<html><body>line 1<br>line 2</body></html>", StringUtils.normalizeHTML("<html><body>line 1<br></body></html><html><body>line 2</body></html>"));
    }

    public void testToHexString()
    {
        assertEquals("", StringUtils.toHexString(new byte[0], ' '));
        assertEquals("0A", StringUtils.toHexString(new byte[]{0x0a}, ' '));
        assertEquals("12 0A", StringUtils.toHexString(new byte[]{0x12, 0x0a}, ' '));
        assertEquals("120A", StringUtils.toHexString(new byte[]{0x12, 0x0a}, '\0'));
    }

    public void testToList()
    {
        assertEquals(Collections.EMPTY_LIST, StringUtils.toList(""));
        assertEquals(Arrays.asList((String) null), StringUtils.toList(StringUtils.NULL));
        assertEquals(Arrays.asList("null"), StringUtils.toList("null"));

        assertEquals(StringUtils.toList("item"), Arrays.asList("item"));
        assertEquals(Arrays.asList("item", "item2"), StringUtils.toList("item, item2"));

        assertEquals(Arrays.asList("item", "item, 2"), StringUtils.toList("item, item\\, 2"));
        assertEquals(Arrays.asList("item", "item", "2"), StringUtils.toList("item, item, 2"));
    }

    public void testIsEmpty()
    {
        assertTrue(StringUtils.isEmpty(null));
        assertTrue(StringUtils.isEmpty(""));
        assertFalse(StringUtils.isEmpty("  "));
        assertFalse(StringUtils.isEmpty("string"));
    }
}
