/**
 * Copyright 2004 by jFramework.org.
 */
package org.jframework.core.util;

import java.io.*;
import java.lang.reflect.Array;
import java.util.ArrayList;
import java.util.List;
import java.net.URL;

/**
 * All things JUnit.
 *
 * @author mgridnev
 * @version 2.0, Mar 3, 2007
 */
public abstract class TestUtils
{
    /**
     * Returns a file that is guaranted to be on the class path and therefore open successfully with {@link
     * ClassLoader#getSystemResourceAsStream(String)}
     *
     * @param fileName relative file name
     * @return a file on the class path
     */
    public static String getClassLoaderReachableFilePath(String fileName)
    {
        final StackTraceElement[] elements = Thread.currentThread().getStackTrace();

        int index = 0;
        for (StackTraceElement element : elements) {
            if (TestUtils.class.getName().equals(element.getClassName())) {
                break;
            }

            index ++;
        }

        final Class clsCaller;
        try {
            clsCaller = Class.forName(elements[index + 1].getClassName());
        } catch (ClassNotFoundException e) {
            throw new RuntimeException(e);
        }

        final String strClassName = clsCaller.getName();

        // Convert to relative class file path
        final String strRelativeClassPath = strClassName.replace('.', '/') + ".class";

        // Convert to absolute path
        final ClassLoader loader = clsCaller.getClassLoader();

        final URL url = loader.getResource(strRelativeClassPath);
        final String strAbsoluteClassPath = url.getPath();

        // Using file absolute class path as a base, arrive at desired file path
        return strAbsoluteClassPath.substring(0, strAbsoluteClassPath.length() -
            strRelativeClassPath.length()) + fileName;
    }

    /**
     * Creates a populated file.
     *
     * @param file file to create
     * @param in   file contents
     */
    public static void createFile(File file, InputStream in) throws IOException
    {
        final OutputStream out = new FileOutputStream(file);

        try {
            while (in.available() > 0) {
                byte[] rg = new byte[in.available()];
                out.write(rg, 0, in.read(rg));
            }
        } finally {
            out.close();
        }
    }

    /**
     * Checkes whether the object is an array
     *
     * @param object the object
     * @return <code>true</code> if object is an array or is <code>null</code>, false otherwise
     */
    public static boolean isArray(Object object)
    {
        return object != null && object.getClass().isArray();
    }

    /**
     * Converts an array of possibly primite types to a list
     *
     * @param object an array
     * @return a list of objects
     * @throws IllegalArgumentException if object is not an array
     */
    public static List asList(Object object) throws IllegalArgumentException
    {
        List<Object> list = new ArrayList<Object>();

        if (object == null) {
            return list;
        }

        if (!object.getClass().isArray()) {
            throw new IllegalArgumentException("Not an array: " + Object.class.getName());
        }

        final int size = Array.getLength(object);
        for (int i = 0; i < size; i++) {
            list.add(Array.get(object, i));
        }

        return list;
    }
}
